#ifndef GAME_H
#define GAME_H

#include <map>
#include <vector>
#include <sstream>
#include <string>
#include "SDL/SDL_thread.h"

class classPlayer;
class classnpc;


#include "classplayer.h"
#include "classnpc.h"
#include "sceneslib.h"
#include "stage.h"
#include "objects/object.h"
#include "class_config.h"
#include "scenes/dialogs.h"



class game
{
public:
    game();
	~game();
	void initGame();
	bool showIntro();
    void quick_load_game();
	void showGame(bool can_characters_move=true);
	int getMapPointLock(struct st_position);
	void framerate_limiter();
	st_position checkScrolling();
	void fps_count();
	void horizontal_transition_screen(int tileX, int tileY, bool is_door, character* playerObj);
	void horizontal_screen_move(short direction, bool is_door, short tileX, short tileY);
	void got_weapon();
	void show_ending();
	void draw_explosion(short int centerX, short int centerY, bool show_players);
	void draw_implosion(short int centerX, short int centerY);
	// PLAYER methods
	void show_player(short int player_n);
	void set_player_position(st_position pos, short int player_n);
	void change_player_position(short xinc, short yinc, short player_n);
	void set_player_anim_type(short int anim_type, short int player_n);
	st_position get_player_position(short int player_n);
	st_size get_player_size(short int player_n);
	void set_player_direction(short int direction, short int player_n);
	void player_fall(short int player_n);
	void boss_intro();
	object* get_player_platform();
	void check_player_return_teleport(); // if player is inside a teleporter, killing a boss makes him teleport out
	bool must_show_boss_hp();
	void fill_boss_hp_bar();
	void reset_stage_npcs();

	// thread methods
	void init_threads();
	void th_move_npcs();

private:
	void load_map_links();
	bool test_teleport(classPlayer *test_player);
	void remove_all_projectiles();
	void debug_draw_map_links();
	void start_stage();
	void restart_stage();
	void load_game_objects();
	std::string cent_number(int n);
	void transitionScreen(short int type, short int map_n, short int adjust_x, classPlayer *pObj);
	void set_current_map(int);
	short int get_current_map();
	void walk_character_to_screen_point_x(character* char_obj, short pos_x); // keeps walking (and jumping obstacles) until reaching a given point in screen (not in map, that should have its own function for that)
	void set_player_teleporter(short set_teleport_n, st_position set_player_pos);
	bool is_player_on_teleporter();
	used_teleporter get_player_teleporter();
	void finish_player_teleporter();


public:
	int currentStage;
	bool _run_move_npcs_thread;


private:
	stage* loaded_stage;
	std::vector<classPlayer> players;
	scenesLib scenes;
	unsigned int fps_timer;
	int fps_counter;
	std::stringstream fps_msg;
	struct format_v1_0::st_game_config config;
	std::vector<struct format_v1_0::st_map_link> map_link_list;
	st_position selected_stage;
	std::map<std::string, object> game_object_list;
	class_config config_manager;
	dialogs game_dialogs;
	// framerate timers
	int _t1;
	float _frame_duration;
	std::map<short, bool> _skull_castle1_used_teleporters; // list of used teleportes (they do not work anymore after added to this list)
	used_teleporter _player_teleporter;
	bool _show_boss_hp; // after set to true, will keep showing the boss HP bar on screen right side

	SDL_Thread *_thread_npc_move;
	SDL_Thread *_thread_object_move;
	SDL_Thread *_thread_player_move;
	SDL_Thread *_thread_colorcycle;

};

#endif // GAME_H
